==> gtkatlantic_0.1.2.tar.bz2: gtkatlantic_0.1.2/src/callback.c <==
/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

#include "global.h"
#include "callback.h"
#include "client.h"
#include "game.h"
#include "interface.h"

#include "engine.h"


/* **** Show all monopd servers */
void Callback_ShowMetaServer(GtkWidget* button)  {

	guint16 connectid;
	gchar *sendstr;

	if(! server_connect(&connectid, config->metaserver_host, config->metaserver_port) ) return;
	connection[connectid]->type = CONNECT_TYPE_META_GETLIST;

	sendstr = g_strdup("GET / HTTP/1.1\n");
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);

	sendstr = g_strdup_printf("Host: %s\n\n", config->metaserver_host);
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);
}


/* **** Get games from all monopd server */
void Callback_RefreshGameMetaServer(GtkWidget* button)  {

	guint16 connectid;
	gchar *sendstr;

	if(! server_connect(&connectid, config->metaserver_host, config->metaserver_port) ) return;
	connection[connectid]->type = CONNECT_TYPE_META_GETGAME;

	sendstr = g_strdup("GET / HTTP/1.1\n");
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);

	sendstr = g_strdup_printf("Host: %s\n\n", config->metaserver_host);
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);
}


/* **** Get games from another server */
void Callback_GetgamesButton(GtkWidget* button, GtkWidget **data)  {

	guint32 port;
	guint16 connectid;
	gchar *host;

	host = g_strstrip( g_strdup( gtk_entry_get_text( GTK_ENTRY(data[0]) ) ) );
	port = atol( gtk_entry_get_text( GTK_ENTRY(data[1]) ) );

	if(! server_connect(&connectid, host, port) ) return;
	connection[connectid]->type = CONNECT_TYPE_MONOPD_GETGAME;

	g_free(host);
}


/* get games: event "row_select" */
void Callback_GetGame_Select(GtkWidget* clist, gint row)  {

	gchar **ptrtxt, *texts[10];
	guint32 i;


	if(global->mode_gameclist != GAMECLIST_MODE_GAMELIST)  return;

	ptrtxt = g_malloc(8192);

	for(i = 0 ; i < 8 ; i++)  {

		memset(ptrtxt, 0, 8192);
		gtk_clist_get_text(GTK_CLIST(clist), row, i, ptrtxt);
		texts[i] = g_strdup(*ptrtxt);
	}
	g_free(ptrtxt);

	if(global->selected_host)  g_free(global->selected_host);
	if(global->selected_gametype)  g_free(global->selected_gametype);

	global->selected_host = g_strdup( texts[0] );
	global->selected_port = atoi( texts[5] );
	global->selected_gametype = g_strdup( texts[6] );
	global->selected_gameid = atoi( texts[7] );

	for(i = 0 ; i < 8 ; i++)
		g_free(texts[i]);
}


/* get games: event "row_unselect" */
void Callback_GetGame_Unselect(GtkWidget* clist, gint row)  {

	if(global->selected_host)  g_free(global->selected_host);
	if(global->selected_gametype)  g_free(global->selected_gametype);
	global->selected_host = 0;
	global->selected_port = 0;
	global->selected_gametype = 0;
	global->selected_gameid = 0;
}


/* **** Join/create a game */
void Callback_ConnectButton(GtkWidget* clist)  {

	guint16 connectid;
	gchar *text;

	if(!global->selected_host) return;
	if(!global->selected_port) return;
	if(!global->selected_gametype) return;
	if(!global->selected_gameid) return;

	if(! server_connect(&connectid, global->selected_host, global->selected_port) )   return;
	connection[connectid]->type = CONNECT_TYPE_MONOPD_GAME;

	/* set nickname */
	text = g_strdup_printf(".n%s\n", config->nickname);
	client_send(connectid, text, strlen(text));
	g_free(text);

	game_alloc();
	game->connectid = connectid;

	/* create a game */
	if(global->selected_gameid < 0)  {

		game->type = GAME_TYPE_CREATE;
		text = g_strdup_printf(".gn%s\n", global->selected_gametype);
		client_send(connectid, text, strlen(text));
		g_free(text);
	}

	/* join a game */
	if(global->selected_gameid > 0)  {

		game->type = GAME_TYPE_JOIN;
		text = g_strdup_printf(".gj%d\n", global->selected_gameid);
		client_send(connectid, text, strlen(text));
		g_free(text);
	}

	/* get playerlist (fix monopd bug) */
	text = g_strdup(".gp\n");
	client_send(connectid, text, strlen(text));
	g_free(text);

	g_free(global->selected_host);
	g_free(global->selected_gametype);
	global->selected_host = 0;
	global->selected_port = 0;
	global->selected_gametype = 0;
	global->selected_gameid = 0;
}


/* **** Start the game :) */
void Callback_StartButton(GtkWidget* button)  {

	client_send(game->connectid, ".gs\n", 4);
}


/* **** Leave the game */
void Callback_LeaveButton(GtkWidget* button)  {

	server_disconnect(game->connectid);
	g_free(game);
	interface_create_getgamespage();
}


/* **** Enter under the real program */
void Callback_EnterButton(GtkWidget* button)  {

	frame_destroy(global->logo_frame);
	g_free(data->pngfile_logo_image);
	g_free(data->pngfile_logo_alpha);

	interface_create_getgamespage();
}


/* *** send data to server */
void Callback_SendButton(GtkWidget* Button, GtkWidget **Data) {

	guint32 length;
	guchar *tmp;

	tmp = g_strdup( gtk_entry_get_text( GTK_ENTRY(Data[0]) ) );
	tmp = g_strstrip(tmp);
	tmp = g_strconcat(tmp, "\n", NULL);
	length = strlen(tmp);

	client_send(game->connectid, tmp, length);
	g_free(tmp);

	gtk_entry_set_text(GTK_ENTRY(Data[0]), "");
}


/* *** Un/Float Server Message Box*/
void CallBack_toggle_float_message(GtkWidget* widget) {

	if(global->phase != PHASE_GAMEPLAY) return;

	game->message_float = !game->message_float;
	interface_message_and_chat_float_unfloat();
}


/* *** Un/Float Chat Box*/
void CallBack_toggle_float_chat(GtkWidget* widget) {

	if(global->phase != PHASE_GAMEPLAY) return;

	game->chat_float = !game->chat_float;
	interface_message_and_chat_float_unfloat();
}


/* *** "delete_event" signal window message */
void CallBack_delete_window_message(GtkWidget* widget) {

	gtk_widget_hide( GTK_WIDGET(global->MessageBox) );
	gtk_widget_reparent(GTK_WIDGET(global->MessageBox), global->Trick);
	gtk_widget_destroy(widget);
	widget = 0;
	game->message_float = 0;
	interface_message_and_chat_float_unfloat();
}


/* *** "delete_event" signal window chat */
void CallBack_delete_window_chat(GtkWidget* widget) {

	gtk_widget_hide( GTK_WIDGET(game->ChatBox) );
	gtk_widget_reparent(GTK_WIDGET(game->ChatBox), global->Trick);
	gtk_widget_destroy(widget);
	widget = 0;
	game->chat_float = 0;
	interface_message_and_chat_float_unfloat();
}


/* *** "destroy" signal window message */
void CallBack_destroy_window_message(GtkWidget* widget) {

}


/* *** "destroy" signal window chat */
void CallBack_destroy_window_chat(GtkWidget* widget) {

}


/* *** button about window */
void CallBack_button_aboutwindow(GtkWidget* button, GtkWidget *aboutwin) {

	gtk_widget_destroy(aboutwin);
}


/* *** button command pressed */
void CallBack_button_command_pressed(GtkWidget* button) {

	guchar *ptr = gtk_object_get_data(GTK_OBJECT(button), "command");

	client_send(game->connectid, ptr, strlen(ptr) );
}


/* *** button command pressed */
void Callback_AuctionButton(GtkWidget* button, GtkWidget* entry)  {

	guint32 length;
	guchar *tosend, *auction, *money;

	auction = gtk_object_get_data(GTK_OBJECT(entry), "auctionid");

	money = g_strdup( gtk_entry_get_text( GTK_ENTRY(entry) ) );
	money = g_strstrip(money);

	tosend = g_strconcat(".ab", auction, ":", money, "\n", NULL);
	length = strlen(tosend);

	client_send(game->connectid, tosend, length);

	g_free(money);
	g_free(tosend);

	gtk_entry_set_text(GTK_ENTRY(entry), "");
}


/* *** callback clic on estate */
void Callback_ClicOnEstate(GtkWidget* widget, GdkEventButton *event)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );
	guchar *text;

	if(event->button == 1)  { /* left clic */

		text = g_strdup_printf(".hb%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		g_free(text);
	}
	if(event->button == 3)  { /* right clic */

		text = g_strdup_printf(".hs%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		g_free(text);
	}
}


/* *** callback enter an estate */
void Callback_EnterAnEstate(GtkWidget* widget)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );

	guint16 frame = game->board_frame;
	guint16 pic = game->estate[id].pic;

	pic_set_bgcolor(frame, pic, 0x000000);
	pic_set_alpha(frame, pic, 170);
}


/* *** callback leave an estate */
void Callback_LeaveAnEstate(GtkWidget* widget)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );

	guint16 frame = game->board_frame;
	guint16 pic = game->estate[id].pic;

	pic_unset_alpha(frame, pic);
}

==> gtkatlantic_0.1.3.tar.bz2: gtkatlantic_0.1.3/src/callback.c <==
/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

#include "global.h"
#include "callback.h"
#include "client.h"
#include "game.h"
#include "interface.h"

#include "engine.h"


/* **** Show all monopd servers */
void Callback_ShowMetaServer(GtkWidget* button)  {

	guint16 connectid;
	gchar *sendstr;

	if(! server_connect(&connectid, config->metaserver_host, config->metaserver_port) ) return;
	connection[connectid]->type = CONNECT_TYPE_META_GETLIST;

	sendstr = g_strdup("GET / HTTP/1.1\n");
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);

	sendstr = g_strdup_printf("Host: %s\n\n", config->metaserver_host);
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);
}


/* **** Get games from all monopd server */
void Callback_RefreshGameMetaServer(GtkWidget* button)  {

	guint16 connectid;
	gchar *sendstr;

	if(! server_connect(&connectid, config->metaserver_host, config->metaserver_port) ) return;
	connection[connectid]->type = CONNECT_TYPE_META_GETGAME;

	sendstr = g_strdup("GET / HTTP/1.1\n");
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);

	sendstr = g_strdup_printf("Host: %s\n\n", config->metaserver_host);
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);
}


/* **** Get games from another server */
void Callback_GetgamesButton(GtkWidget* button, GtkWidget **data)  {

	guint32 port;
	guint16 connectid;
	gchar *host;

	host = g_strstrip( g_strdup( gtk_entry_get_text( GTK_ENTRY(data[0]) ) ) );
	port = atol( gtk_entry_get_text( GTK_ENTRY(data[1]) ) );

	if(! server_connect(&connectid, host, port) ) return;
	connection[connectid]->type = CONNECT_TYPE_MONOPD_GETGAME;

	g_free(host);
}


/* get games: event "row_select" */
void Callback_GetGame_Select(GtkWidget* clist, gint row)  {

	gchar **ptrtxt, *texts[10];
	guint32 i;


	if(global->mode_gameclist != GAMECLIST_MODE_GAMELIST)  return;

	ptrtxt = g_malloc(8192);

	for(i = 0 ; i < 8 ; i++)  {

		memset(ptrtxt, 0, 8192);
		gtk_clist_get_text(GTK_CLIST(clist), row, i, ptrtxt);
		texts[i] = g_strdup(*ptrtxt);
	}
	g_free(ptrtxt);

	if(global->selected_host)  g_free(global->selected_host);
	if(global->selected_gametype)  g_free(global->selected_gametype);

	global->selected_host = g_strdup( texts[0] );
	global->selected_port = atoi( texts[5] );
	global->selected_gametype = g_strdup( texts[6] );
	global->selected_gameid = atoi( texts[7] );

	for(i = 0 ; i < 8 ; i++)
		g_free(texts[i]);
}


/* get games: event "row_unselect" */
void Callback_GetGame_Unselect(GtkWidget* clist, gint row)  {

	if(global->selected_host)  g_free(global->selected_host);
	if(global->selected_gametype)  g_free(global->selected_gametype);
	global->selected_host = 0;
	global->selected_port = 0;
	global->selected_gametype = 0;
	global->selected_gameid = 0;
}


/* **** Join/create a game */
void Callback_ConnectButton(GtkWidget* clist)  {

	guint16 connectid;
	gchar *text;

	if(!global->selected_host) return;
	if(!global->selected_port) return;
	if(!global->selected_gametype) return;
	if(!global->selected_gameid) return;

	if(! server_connect(&connectid, global->selected_host, global->selected_port) )   return;
	connection[connectid]->type = CONNECT_TYPE_MONOPD_GAME;

	/* set nickname */
	text = g_strdup_printf(".n%s\n", config->nickname);
	client_send(connectid, text, strlen(text));
	g_free(text);

	game_alloc();
	game->connectid = connectid;

	/* create a game */
	if(global->selected_gameid < 0)  {

		game->type = GAME_TYPE_CREATE;
		text = g_strdup_printf(".gn%s\n", global->selected_gametype);
		client_send(connectid, text, strlen(text));
		g_free(text);
	}

	/* join a game */
	if(global->selected_gameid > 0)  {

		game->type = GAME_TYPE_JOIN;
		text = g_strdup_printf(".gj%d\n", global->selected_gameid);
		client_send(connectid, text, strlen(text));
		g_free(text);
	}

	/* get playerlist (fix monopd bug) */
	text = g_strdup(".gp\n");
	client_send(connectid, text, strlen(text));
	g_free(text);

	g_free(global->selected_host);
	g_free(global->selected_gametype);
	global->selected_host = 0;
	global->selected_port = 0;
	global->selected_gametype = 0;
	global->selected_gameid = 0;
}


/* **** Start the game :) */
void Callback_StartButton(GtkWidget* button)  {

	client_send(game->connectid, ".gs\n", 4);
}


/* **** Leave the game */
void Callback_LeaveButton(GtkWidget* button)  {

	server_disconnect(game->connectid);
	g_free(game);
	interface_create_getgamespage();
}


/* **** Enter in the real program */
void Callback_EnterButton(GtkWidget* button)  {

	frame_destroy(global->logo_frame);
	g_free(data->pngfile_logo_image);
	g_free(data->pngfile_logo_alpha);

	interface_create_getgamespage();
}


/* *** send data to server */
void Callback_SendButton(GtkWidget* Button, GtkWidget **Data) {

	guint32 length;
	guchar *tmp;

	tmp = g_strdup( gtk_entry_get_text( GTK_ENTRY(Data[0]) ) );
	tmp = g_strstrip(tmp);
	tmp = g_strconcat(tmp, "\n", NULL);
	length = strlen(tmp);

	client_send(game->connectid, tmp, length);
	g_free(tmp);

	gtk_entry_set_text(GTK_ENTRY(Data[0]), "");
}


/* *** Un/Float Server Message Box*/
void CallBack_toggle_float_message(GtkWidget* widget) {

	if(global->phase != PHASE_GAMEPLAY) return;

	game->message_float = !game->message_float;
	interface_message_and_chat_float_unfloat();
}


/* *** Un/Float Chat Box*/
void CallBack_toggle_float_chat(GtkWidget* widget) {

	if(global->phase != PHASE_GAMEPLAY) return;

	game->chat_float = !game->chat_float;
	interface_message_and_chat_float_unfloat();
}


/* *** "delete_event" signal window message */
void CallBack_delete_window_message(GtkWidget* widget) {

	gtk_widget_hide( GTK_WIDGET(global->MessageBox) );
	gtk_widget_reparent(GTK_WIDGET(global->MessageBox), global->Trick);
	gtk_widget_destroy(widget);
	widget = 0;
	game->message_float = 0;
	interface_message_and_chat_float_unfloat();
}


/* *** "delete_event" signal window chat */
void CallBack_delete_window_chat(GtkWidget* widget) {

	gtk_widget_hide( GTK_WIDGET(game->ChatBox) );
	gtk_widget_reparent(GTK_WIDGET(game->ChatBox), global->Trick);
	gtk_widget_destroy(widget);
	widget = 0;
	game->chat_float = 0;
	interface_message_and_chat_float_unfloat();
}


/* *** "destroy" signal window message */
void CallBack_destroy_window_message(GtkWidget* widget) {

}


/* *** "destroy" signal window chat */
void CallBack_destroy_window_chat(GtkWidget* widget) {

}


/* *** button about window */
void CallBack_button_aboutwindow(GtkWidget* button, GtkWidget *aboutwin) {

	gtk_widget_destroy(aboutwin);
}


/* *** button command pressed */
void CallBack_button_command_pressed(GtkWidget* button) {

	guchar *ptr = gtk_object_get_data(GTK_OBJECT(button), "command");

	client_send(game->connectid, ptr, strlen(ptr) );
}


/* *** button command pressed */
void Callback_AuctionButton(GtkWidget* button, GtkWidget* entry)  {

	guint32 length;
	guchar *tosend, *auction, *money;

	auction = gtk_object_get_data(GTK_OBJECT(entry), "auctionid");

	money = g_strdup( gtk_entry_get_text( GTK_ENTRY(entry) ) );
	money = g_strstrip(money);

	tosend = g_strconcat(".ab", auction, ":", money, "\n", NULL);
	length = strlen(tosend);

	client_send(game->connectid, tosend, length);

	g_free(money);
	g_free(tosend);

	gtk_entry_set_text(GTK_ENTRY(entry), "");
}


/* *** callback clic on estate */
void Callback_ClicOnEstate(GtkWidget* widget, GdkEventButton *event)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );
	guchar *text;

	if(event->button == 1)  { /* left clic -> build house */

		text = g_strdup_printf(".hb%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		g_free(text);
	}
	if(event->button == 3)  { /* right clic -> sell house */

		text = g_strdup_printf(".hs%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		free(text);
	}
	if(event->button == 2)  { /* middle clic -> toggle_mortgage */

		text = g_strdup_printf(".em%d\n", id);
		client_send(game->connectid, text, strlen(text) );
		g_free(text);
	}
}


/* *** callback enter an estate */
void Callback_EnterAnEstate(GtkWidget* widget)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );

	guint16 frame = game->board_frame;
	guint16 pic = game->estate[id].pic;

	pic_set_bgcolor(frame, pic, 0x000000);
	pic_set_alpha(frame, pic, 170);
}


/* *** callback leave an estate */
void Callback_LeaveAnEstate(GtkWidget* widget)  {

	guint8 id = atoi( gtk_object_get_data(GTK_OBJECT(widget), "estateid") );

	guint16 frame = game->board_frame;
	guint16 pic = game->estate[id].pic;

	pic_unset_alpha(frame, pic);
}
